local DEBUG = nil
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local L_CALL = "%s looted %s!"
local L_CALL_WITH_SOURCE = "%s looted %s (%s)!"

local ignoredItems = {
	[30316] = true, -- Devastation
	[30317] = true, -- Cosmic Infuser
	[30312] = true, -- Infinity Blade
	[30318] = true, -- Netherstrand Longbow
	[30313] = true, -- Staff of Disintegration
	[30311] = true, -- Warp Slicer
	[30320] = true, -- Bundle of Nether Spikes
	[30314] = true, -- Phaseshift Bulwark
	[30319] = true, -- Nether Spikes
	[29434] = true, -- Badge of Justice
	[22450] = true, -- Void Crystal
	[20725] = true, -- Nexus Crystal

	[32428] = true, -- Heart of Darkness

	[32230] = true, -- Shadowsong Amethyst
	[32249] = true, -- Seaspray Emerald
	[32231] = true, -- Pyrestone
	[32229] = true, -- Lionseye
	[32228] = true, -- Empyrean Sapphire
	[32227] = true, -- Crimson Spinel

	[36919] = true, -- Cardinal Ruby
	[36925] = true, -- Majestic Zircon
	[36922] = true, -- King's Amber
	[36928] = true, -- Dreadstone
	[36934] = true, -- Eye of Zul
	[36931] = true, -- Ametrine

	[34057] = true, -- Abyss Crystal
	[52722] = true, -- Maelstrom Crystal
}

local pname = UnitName("player")
local rollItems = {}

local shouldTransmit = true
local f = CreateFrame("Frame")

local lootStrings = {
	LOOT_ITEM:gsub("%%s", "(.+)"),
	LOOT_ITEM_MULTIPLE:gsub("%%s", "(.+)"),
	LOOT_ITEM_PUSHED_SELF:gsub("%%s", "(.+)"),
	LOOT_ITEM_PUSHED_SELF_MULTIPLE:gsub("%%s", "(.+)"),
	LOOT_ITEM_SELF:gsub("%%s", "(.+)"),
	LOOT_ITEM_SELF_MULTIPLE:gsub("%%s", "(.+)"),
	--LOOT_ITEM_CREATED_SELF:gsub("%%s", "(.+)"),
	--LOOT_ITEM_CREATED_SELF_MULTIPLE:gsub("%%s", "(.+)"),
	--CREATED_ITEM:gsub("%%s", "(.+)"),
	--CREATED_ITEM_MULTIPLE:gsub("%%s", "(.+)"),
}

local candidates = {}
local totalElapsed = 0
local function onUpdate(self, elapsed)
	totalElapsed = totalElapsed + elapsed
	if totalElapsed > 10 then
		self:SetScript("OnUpdate", nil)
		totalElapsed = 0
		if #candidates > 0 then
			table.sort(candidates)
			shouldTransmit = candidates[1] == pname and true or false
			wipe(candidates)
		else
			shouldTransmit = true
		end
	end
end
local function checkGroup()
	local i = select(2, IsInInstance())
	if i == "pvp" or i == "arena" then return end
	f:SetScript("OnUpdate", onUpdate)
	SendAddonMessage("BTC", "ping", "RAID")
	return true
end

local function getSource(looter)
	if UnitInRaid("player") then
		for i = 1, GetNumRaidMembers() do
			local u = "raid" .. i .. "target"
			if UnitExists(u) and UnitIsDead(u) and UnitClassification(u) == "worldboss" then
				return UnitName(u)
			end
		end
	else
		local unit = looter .. "-target"
		if UnitExists(unit) and UnitIsDead(unit) and UnitIsEnemy(looter, unit) then
			return UnitName(unit)
		end
	end
end

local function checkArgs(first, second)
	if second then return first, second
	else return pname, first end
end

local function transmit(msg)
	if shouldTransmit then
		SendChatMessage(msg, "GUILD")
	elseif DEBUG then
		print(msg)
	end
end

local function notify(msg)
	local player, item = nil, nil
	for i, v in next, lootStrings do
		local f, s = checkArgs(msg:match(v))
		if f and s then
			player, item = f, s
			break
		end
	end
	if not item or not player then return 1, msg, player, item end
	local name, link, quality = GetItemInfo(item)
	if not name then return 6, msg, item end
	if not quality then return 2, item end
	if quality < 4 then return 3, quality, item end
	local itemId = select(3, link:find("item:(%d+):"))
	if type(tonumber(itemId)) ~= "number" then return 4, item, itemId end
	itemId = tonumber(itemId)
	if ignoredItems[itemId] then return 5, itemId end
	local source = getSource(player)
	if source then
		transmit(L_CALL_WITH_SOURCE:format(player, link, source))
	else
		transmit(L_CALL:format(player, link))
	end
end

function f:START_LOOT_ROLL(id)
	local _, name, _, quality = GetLootRollItemInfo(id)
	local _, link, rarity = GetItemInfo(name)
	if not link or not rarity then
		print(name, "no link or rarity on roll item")
		return
	end
	if not rarity or rarity < 4 then
		print(tostring(rarity), tostring(quality), name)
		return
	end
	local itemId = select(3, link:find("item:(%d+):"))
	if type(tonumber(itemId)) ~= "number" then
		print(tostring(itemId), link)
		return
	end
	itemId = tonumber(itemId)
	if ignoredItems[itemId] then
		print(itemId, "ignored")
		return
	end
	rollItems[link] = true
	print("Dropped", tostringall(name, quality, rarity, link))
end

local debugMessages = {
	"The loot message %q was not parsed properly (%s, %s).",
	"Item quality for %q could not be determined.",
	"Quality %d for item %q was lower than 4, and has been ignored.",
	"Item ID could not be determined for %q (%s).",
	"Item ID %d is ignored automatically.",
	"Loot message %q was parsed to %q, but GetItemInfo returned nil.",
}
local function evaluate(returnCode, ...)
	if not DEBUG then return end
	if not returnCode then
		print("evaluate() invoked without a return code.")
		return
	end
	print(debugMessages[returnCode]:format(tostringall(...)))
end

function f:CHAT_MSG_LOOT(msg)
	if not DEBUG and not shouldTransmit then return end
	evaluate(notify(msg))
end

function f:CHAT_MSG_ADDON(prefix, message, distribution, sender)
	if prefix ~= "BTC" then return end
	if message == "ping" then
		SendAddonMessage("BTC", "pong", "RAID")
	end
	candidates[#candidates+1] = sender
end

function f:ADDON_LOADED(addon)
	if addon ~= "BootyCall" then return end

	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	--self:RegisterEvent("START_LOOT_ROLL")

	self:UnregisterEvent("ADDON_LOADED")
end

do
	local lastSize = 0
	function f:RAID_ROSTER_UPDATE()
		local size = UnitInRaid("player") and GetNumRaidMembers() or GetNumPartyMembers()
		if size == lastSize then return end
		if not checkGroup() then return end
		lastSize = size
	end
	f.PARTY_MEMBERS_CHANGED = f.RAID_ROSTER_UPDATE
end

f:SetScript("OnEvent", function(self, event, ...)
	if not self[event] then
		print(event, tostringall(...))
	else
		self[event](self, ...)
	end
end)

f:RegisterEvent("ADDON_LOADED")
f:Show()

